package net.quickpay.paymentexample;

import android.app.Activity;
import android.content.Intent;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.View;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.android.volley.Response;
import com.android.volley.VolleyError;

import net.quickpay.library.DefaultHeaders;
import net.quickpay.library.QuickPayHeaders;
import net.quickpay.library.createPayment.CreatePaymentParameters;
import net.quickpay.library.createPayment.CreatePaymentRequest;
import net.quickpay.library.generatePaymentLink.GeneratePaymentLinkParameters;
import net.quickpay.library.generatePaymentLink.GeneratePaymentLinkRequest;
import net.quickpay.library.QuickPayActivity;
import net.quickpay.library.getPayment.GetPaymentRequest;
import net.quickpay.library.objects.Payment;
import net.quickpay.library.objects.Shipping;
import net.quickpay.library.objects.ShippingMethod;

import org.json.JSONObject;

public class MainActivity extends AppCompatActivity {
    public final String merchantToken = "99357254255b6cc17e9474e8d24c9e550a8504760ab3df537a5a09192e97b63e";
    private int lastCreatedRequestId = 0;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
    }

    private void requestToCreatePaymentWith(String orderId) {
        QuickPayHeaders headers = new DefaultHeaders()
                .setAcceptVersion(QuickPayActivity.DEFAULT_VERSION)
                .setAuthorization(merchantToken);

        CreatePaymentParameters params = new CreatePaymentParameters()
                .setOrderId(orderId)
                .setCurrency("Dkk");

        Shipping sh = new Shipping();
        sh.setMethod(ShippingMethod.STORE_PICK_UP);
        sh.setCompany("YourCompany AS");
        params.setShipping(sh);

        CreatePaymentRequest req = new CreatePaymentRequest(this.getApplicationContext(), headers, params);

        req.executeRequest(new Response.Listener<JSONObject>() {
            @Override
            public void onResponse(JSONObject response) {
                TextView tw = (TextView) findViewById(R.id.createPaymentResultId);
                EditText tw2 = (EditText) findViewById(R.id.transactionIdInput);
                tw.setText(response.optString("id"));
                tw2.setText(response.optString("id"));

                lastCreatedRequestId = Integer.parseInt(response.optString("id"));

                /*
                    Small example of how to extract information through provided classes,
                    instead of dealing with JSONObject directly.
                 */
                Payment payment = new Payment(response);
                String method = "Shipping method: " + payment.getShipping().getMethod();
                Toast toast = Toast.makeText(getApplicationContext(), method, Toast.LENGTH_LONG);
                toast.show();

            }
        }, new Response.ErrorListener() {

            @Override
            public void onErrorResponse(VolleyError error) {

            }
        });
    }

    /**
     * Create a Payment from the entered order id.
     *
     * @param v
     */
    public void onButtonClick(View v) {
        EditText t = (EditText) findViewById(R.id.createPaymentOrderNumber);
        this.requestToCreatePaymentWith(t.getText().toString());
    }


    /**
     * Generate the link to proceed with the payment, and open the QuickPayActivity.
     *
     * @param v
     */
    public void onPayButtonClick(View v) {
        EditText tw2 = (EditText) findViewById(R.id.transactionIdInput);
        String t = tw2.getText().toString();
        this.requestToGenerateLinkWith(t);
    }

    private void requestToGenerateLinkWith(String paymentId) {
        final Activity mainActivity = this;

        GeneratePaymentLinkParameters params = new GeneratePaymentLinkParameters();
        params.setId(Integer.parseInt(paymentId))
                .setAmount(100);

        QuickPayHeaders headers = new DefaultHeaders()
                .setAcceptVersion(QuickPayActivity.DEFAULT_VERSION)
                .setAuthorization(this.merchantToken);

        GeneratePaymentLinkRequest req = new GeneratePaymentLinkRequest(this.getApplicationContext(), headers, params);

        req.executeRequest(new Response.Listener<JSONObject>() {
            @Override
            public void onResponse(JSONObject response) {
                QuickPayActivity.openQuickPayPaymentURL(mainActivity, response.optString("url"));
            }
        }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {

            }
        });
    }

    public void requestPayment(int paymentId){
        // retrieve the payment itself, for example to check
        QuickPayHeaders qph = new DefaultHeaders();
        qph.setAcceptVersion(QuickPayActivity.DEFAULT_VERSION).setAuthorization(merchantToken);
        GetPaymentRequest gp = new GetPaymentRequest(paymentId, qph);

        gp.executeRequest(new Response.Listener<JSONObject>() {

            @Override
            public void onResponse(JSONObject response) {
                Payment p = new Payment(response);
                boolean isAccepted = p.isAccepted();
                Toast toast = Toast.makeText(getApplicationContext(), "Accepted: " + isAccepted, Toast.LENGTH_LONG);
                toast.show();
            }

        }, null);
    }

    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == QuickPayActivity.QUICKPAY_INTENT_CODE) {
            if (resultCode == RESULT_OK) {

                String returnedResult = data.getData().toString();
                if (returnedResult.equals(QuickPayActivity.SUCCESS_RESULT)) {
                    requestPayment(lastCreatedRequestId);

                } else if (returnedResult.equals(QuickPayActivity.CANCEL_RESULT)) {
                    Toast toast = Toast.makeText(this, "Result: " + returnedResult, Toast.LENGTH_LONG);
                    toast.show();
                }

            } else if (resultCode == RESULT_CANCELED) {
                Toast toast = Toast.makeText(this, "The user cancelled out of the activity", Toast.LENGTH_LONG);
                toast.show();
            }
        }
    }
}
